@@ -108,6 +108,7 @@ struct _GrEditPage
         GtkWidget *add_step_button;
         GtkWidget *link_image_button;
         GtkWidget *timer_button;
+        GtkWidget *temperature_button;
         GtkWidget *prev_step_button;
         GtkWidget *next_step_button;
         GtkWidget *image_popover;
@@ -117,6 +118,7 @@ struct _GrEditPage
         GtkWidget *temperature_spin;
         GtkWidget *celsius_button;
         GtkWidget *timer_spin;
+        GtkWidget *timer_title;
         GtkWidget *preview_stack;
 
         GrRecipeSearch *search;
@@ -1173,15 +1175,22 @@ time_spin_activate (GtkEntry *entry, GrEditPage *self)
         double minutes;
         double seconds;
         g_autofree char *text = NULL;
+        const char *title;
 
-        gtk_spin_button_update (GTK_SPIN_BUTTON (entry));
+        gtk_spin_button_update (GTK_SPIN_BUTTON (self->timer_spin));
 
-        adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (entry));
+        adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (self->timer_spin));
         hours = gtk_adjustment_get_value (adjustment) / 3600.0;
         minutes = (hours - floor (hours)) * 60.0;
         seconds = (minutes - floor (minutes)) * 60.0;
 
-        text = g_strdup_printf ("[timer:%02.0f:%02.0f:%02.0f]", floor (hours), floor (minutes), floor (seconds + 0.5));
+        title = gtk_entry_get_text (GTK_ENTRY (self->timer_title));
+
+        if (title && title[0])
+                text = g_strdup_printf ("[timer:%02.0f:%02.0f:%02.0f,%s]", floor (hours), floor (minutes), floor (seconds + 0.5), title);
+        else
+                text = g_strdup_printf ("[timer:%02.0f:%02.0f:%02.0f]", floor (hours), floor (minutes), floor (seconds + 0.5));
+
         add_tag_to_step (self, text);
 
         gtk_popover_popdown (GTK_POPOVER (self->timer_popover));
@@ -1359,25 +1368,27 @@ preview_visible_changed (GrEditPage *page)
                 gtk_widget_set_sensitive (page->add_step_button, TRUE);
                 gtk_widget_set_sensitive (page->link_image_button, TRUE);
                 gtk_widget_set_sensitive (page->timer_button, TRUE);
+                gtk_widget_set_sensitive (page->temperature_button, TRUE);
                 gtk_widget_set_visible (page->prev_step_button, FALSE);
                 gtk_widget_set_visible (page->next_step_button, FALSE);
+                gr_cooking_view_stop (GR_COOKING_VIEW (page->cooking_view));
         }
         else {
                 g_autoptr(GArray) images = NULL;
                 g_autofree char *instructions = NULL;
-                int index = 0;
 
                 gtk_widget_set_sensitive (page->add_step_button, FALSE);
                 gtk_widget_set_sensitive (page->link_image_button, FALSE);
                 gtk_widget_set_sensitive (page->timer_button, FALSE);
+                gtk_widget_set_sensitive (page->temperature_button, FALSE);
                 gtk_widget_set_visible (page->prev_step_button, TRUE);
                 gtk_widget_set_visible (page->next_step_button, TRUE);
 
                 g_object_get (page->images, "images", &images, NULL);
                 instructions = get_text_view_text (GTK_TEXT_VIEW (page->instructions_field));
 
-                gr_cooking_view_set_images (GR_COOKING_VIEW (page->cooking_view), images, index);
-                gr_cooking_view_set_instructions (GR_COOKING_VIEW (page->cooking_view), instructions);
+                gr_cooking_view_set_data (GR_COOKING_VIEW (page->cooking_view), NULL, instructions, images);
+                gr_cooking_view_start (GR_COOKING_VIEW (page->cooking_view));
 
                 update_steppers (page);
         }
@@ -1386,24 +1397,14 @@ preview_visible_changed (GrEditPage *page)
 static void
 prev_step (GrEditPage *page)
 {
-        int step;
-
-        step = gr_cooking_view_get_step (GR_COOKING_VIEW (page->cooking_view)) - 1;
-        if (step >= 0)
-                gr_cooking_view_set_step (GR_COOKING_VIEW (page->cooking_view), step);
-
+        gr_cooking_view_prev_step (GR_COOKING_VIEW (page->cooking_view));
         update_steppers (page);
 }
 
 static void
 next_step (GrEditPage *page)
 {
-        int step;
-
-        step = gr_cooking_view_get_step (GR_COOKING_VIEW (page->cooking_view)) + 1;
-        if (step < gr_cooking_view_get_n_steps (GR_COOKING_VIEW (page->cooking_view)))
-                gr_cooking_view_set_step (GR_COOKING_VIEW (page->cooking_view), step);
-
+        gr_cooking_view_next_step (GR_COOKING_VIEW (page->cooking_view));
         update_steppers (page);
 }
 
@@ -1461,6 +1462,7 @@ gr_edit_page_class_init (GrEditPageClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, add_step_button);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, link_image_button);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, timer_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, temperature_button);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, prev_step_button);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, next_step_button);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, image_popover);
@@ -1470,6 +1472,7 @@ gr_edit_page_class_init (GrEditPageClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, temperature_spin);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, celsius_button);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, timer_spin);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, timer_title);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, preview_stack);
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, cooking_view);
 
@@ -2096,6 +2099,7 @@ gr_edit_page_save (GrEditPage *page)
                               "instructions", instructions,
                               "diets", diets,
                               "images", images,
+                              "mtime", g_date_time_new_now_utc (),
                               NULL);
                 ret = gr_recipe_store_update_recipe (store, page->recipe, old_id, &error);
         }
