@@ -33,6 +33,38 @@
 #include "gr-app.h"
 
 
+/**
+ * Data storage
+ * ------------
+ *
+ * The current storage layer is very simple-minded. We just write data out
+ * as a number of key files:
+ * - recipes.db for recipes, with each recipe being its own group
+ * - chefs.db for chefs, with each chef being its own group
+ *
+ * There's a few auxiliary files:
+ * - picks, with lists of things to put on the landing page
+ * - favorites.db, with a list of the user's favorites
+ * - shopping.db, with the current shppping list
+ * - user, containing the user id of the user
+ * - cooking, with a counter for how often cooking mode was launched
+ *
+ * Some of these files are preinstalled (picks.db), some are only per-user
+ * (favorites.db, shopping.db, user, cooking), and some are both, with the
+ * per-user list overlaying the preinstalled one.
+ *
+ * Data from the preinstalled files is treated as readonly unless it
+ * belongs to the current user.
+ *
+ * Data is always written to the per-user files.
+ *
+ * Data at runtime
+ * ---------------
+ *
+ * At runtime, we keep GrRecipe and GrChef objects in two separate hash tables.
+ * Recipes have unique IDs of the form R_<dish_name>_by_<chef_id>.
+ */
+
 struct _GrRecipeStore
 {
         GObject parent;
@@ -342,12 +374,13 @@ load_recipes (GrRecipeStore *self,
                                               "diets", diets,
                                               "images", images,
                                               "default-image", default_image,
+                                              "mtime", mtime,
                                               NULL);
                 }
                 else {
                         gboolean own;
 
-                        own = strcmp (author, self->user) == 0;
+                        own = g_strcmp0 (author, self->user) == 0;
                         recipe = g_object_new (GR_TYPE_RECIPE,
                                                "id", id,
                                                "name", name,
@@ -383,20 +416,25 @@ save_recipes (GrRecipeStore *self)
 {
         g_autoptr(GKeyFile) keyfile = NULL;
         g_autofree char *path = NULL;
-        GHashTableIter iter;
         const char *key;
         GrRecipe *recipe;
         g_autoptr(GError) error = NULL;
-        const char *tmp;
+        const char *dir;
+        GList *keys, *l;
 
         keyfile = g_key_file_new ();
 
-        path = g_build_filename (get_user_data_dir (), "recipes.db", NULL);
+        if (g_getenv ("RECIPES_DATA_DIR"))
+                dir = g_getenv ("RECIPES_DATA_DIR");
+        else
+                dir = get_user_data_dir ();
+        path = g_build_filename (dir, "recipes.db", NULL);
 
         g_message ("Save recipe db: %s", path);
 
-        g_hash_table_iter_init (&iter, self->recipes);
-        while (g_hash_table_iter_next (&iter, (gpointer *)&key, (gpointer *)&recipe)) {
+        keys = g_hash_table_get_keys (self->recipes);
+        keys = g_list_sort (keys, (GCompareFunc)strcmp);
+        for (l = keys; l; l = l->next) {
                 const char *name;
                 const char *author;
                 const char *description;
@@ -419,6 +457,9 @@ save_recipes (GrRecipeStore *self)
                 int i;
                 gboolean readonly;
 
+                key = l->data;
+                recipe = g_hash_table_lookup (self->recipes, key);
+
                 name = gr_recipe_get_name (recipe);
                 author = gr_recipe_get_author (recipe);
                 description = gr_recipe_get_description (recipe);
@@ -440,12 +481,11 @@ save_recipes (GrRecipeStore *self)
 
                 g_object_get (recipe, "images", &images, NULL);
 
-                tmp = get_user_data_dir ();
                 paths = g_new0 (char *, images->len + 1);
                 for (i = 0; i < images->len; i++) {
                         GrImage *ri = &g_array_index (images, GrImage, i);
-                        if (g_str_has_prefix (ri->path, tmp))
-                                paths[i] = g_strdup (ri->path + strlen (tmp) + 1);
+                        if (g_str_has_prefix (ri->path, dir))
+                                paths[i] = g_strdup (ri->path + strlen (dir) + 1);
                         else
                                 paths[i] = g_strdup (ri->path);
                 }
@@ -482,6 +522,8 @@ save_recipes (GrRecipeStore *self)
                 }
         }
 
+        g_list_free (keys);
+
         if (!g_key_file_save_to_file (keyfile, path, &error)) {
                 g_error ("Failed to save recipe database: %s", error->message);
         }
@@ -805,25 +847,33 @@ save_chefs (GrRecipeStore *store)
 {
         g_autoptr(GKeyFile) keyfile = NULL;
         g_autofree char *path = NULL;
-        GHashTableIter iter;
         const char *key;
         GrChef *chef;
         g_autoptr(GError) error = NULL;
-        const char *tmp;
+        const char *dir;
+        GList *keys, *l;
 
         keyfile = g_key_file_new ();
 
-        path = g_build_filename (get_user_data_dir (), "chefs.db", NULL);
+        if (g_getenv ("RECIPES_DATA_DIR"))
+                dir = g_getenv ("RECIPES_DATA_DIR");
+        else
+                dir = get_user_data_dir ();
+        path = g_build_filename (dir, "chefs.db", NULL);
 
         g_message ("Save chefs db: %s", path);
 
-        g_hash_table_iter_init (&iter, store->chefs);
-        while (g_hash_table_iter_next (&iter, (gpointer *)&key, (gpointer *)&chef)) {
+        keys = g_hash_table_get_keys (store->chefs);
+        keys = g_list_sort (keys, (GCompareFunc)strcmp);
+        for (l = keys; l; l = l->next) {
                 const char *name;
                 const char *fullname;
                 const char *description;
                 const char *image_path;
 
+                key = l->data;
+                chef = g_hash_table_lookup (store->chefs, key);
+
                 if (gr_chef_is_readonly (chef))
                         continue;
 
@@ -832,11 +882,10 @@ save_chefs (GrRecipeStore *store)
                 description = gr_chef_get_description (chef);
                 image_path = gr_chef_get_image (chef);
 
-                tmp = get_user_data_dir ();
-                if (image_path && g_str_has_prefix (image_path, tmp)) {
+                if (image_path && g_str_has_prefix (image_path, dir)) {
                         g_autofree char *tmp2 = NULL;
 
-                        tmp2 = g_strdup (image_path + strlen (tmp) + 1);
+                        tmp2 = g_strdup (image_path + strlen (dir) + 1);
                         g_key_file_set_string (keyfile, key, "Image", tmp2);
                 }
                 else
@@ -846,6 +895,7 @@ save_chefs (GrRecipeStore *store)
                 g_key_file_set_string (keyfile, key, "Fullname", fullname ? fullname : "");
                 g_key_file_set_string (keyfile, key, "Description", description ? description : "");
         }
+        g_list_free (keys);
 
         if (!g_key_file_save_to_file (keyfile, path, &error)) {
                 g_error ("Failed to save chefs database: %s", error->message);
@@ -922,6 +972,21 @@ gr_recipe_store_init (GrRecipeStore *self)
 
         load_user (self, user_dir);
 
+        if (g_getenv ("RECIPES_DATA_DIR")) {
+                const char *dir = g_getenv ("RECIPES_DATA_DIR");
+
+                g_message ("Loading data from RECIPES_DATA_DIR: %s", dir);
+
+                load_recipes (self, dir, FALSE);
+                load_chefs (self, dir, FALSE);
+                load_picks (self, dir);
+
+                g_message ("%d recipes loaded", g_hash_table_size (self->recipes));
+                g_message ("%d chefs loaded", g_hash_table_size (self->chefs));
+
+                return;
+        }
+
         /* First load preinstalled data */
         if (!load_recipes (self, data_dir, TRUE))
                 load_recipes (self, uninstalled_dir, TRUE);
@@ -1563,7 +1628,7 @@ gr_recipe_store_has_chef (GrRecipeStore *self,
 
         g_hash_table_iter_init (&iter, self->recipes);
         while (g_hash_table_iter_next (&iter, NULL, (gpointer *)&recipe)) {
-                if (strcmp (gr_chef_get_id (chef), gr_recipe_get_author (recipe)) == 0)
+                if (g_strcmp0 (gr_chef_get_id (chef), gr_recipe_get_author (recipe)) == 0)
                         return TRUE;
         }
 
@@ -1579,7 +1644,7 @@ gr_recipe_store_has_cuisine (GrRecipeStore *self,
 
         g_hash_table_iter_init (&iter, self->recipes);
         while (g_hash_table_iter_next (&iter, NULL, (gpointer *)&recipe)) {
-                if (strcmp (cuisine, gr_recipe_get_cuisine (recipe)) == 0)
+                if (g_strcmp0 (cuisine, gr_recipe_get_cuisine (recipe)) == 0)
                         return TRUE;
         }
 
@@ -1596,6 +1661,8 @@ struct _GrRecipeSearch
 
         char **query;
 
+        GDateTime *timestamp;
+
         gulong idle;
         GHashTableIter iter;
 
@@ -1666,10 +1733,16 @@ static gboolean
 recipe_matches (GrRecipeSearch *search,
                 GrRecipe       *recipe)
 {
+        if (strcmp (search->query[0], "is:any") == 0)
+                return TRUE;
         if (strcmp (search->query[0], "is:favorite") == 0)
                 return gr_recipe_store_is_favorite (search->store, recipe);
         else if (strcmp (search->query[0], "is:shopping") == 0)
                 return gr_recipe_store_is_in_shopping (search->store, recipe);
+        if (g_str_has_prefix (search->query[0], "ct:"))
+                return g_date_time_compare (gr_recipe_get_ctime (recipe), search->timestamp) > 0;
+        else if (g_str_has_prefix (search->query[0], "mt:"))
+                return g_date_time_compare (gr_recipe_get_mtime (recipe), search->timestamp) > 0;
         else
                 return gr_recipe_matches (recipe, (const char **)search->query);
 }
@@ -1715,6 +1788,13 @@ start_search (GrRecipeSearch *search)
                 return;
         }
 
+        if (g_str_has_prefix (search->query[0], "ct:") ||
+            g_str_has_prefix (search->query[0], "mt:")) {
+                const char *time = search->query[0] + strlen ("ct:");
+                g_clear_pointer (&search->timestamp, g_date_time_unref);
+                search->timestamp = date_time_from_string (time);
+        }
+
         if (search->idle == 0) {
                 g_hash_table_iter_init (&search->iter, search->store->recipes);
                 clear_pending (search);
@@ -1863,6 +1943,7 @@ gr_recipe_search_finalize (GObject *object)
         stop_search (search);
         g_strfreev (search->query);
         g_object_unref (search->store);
+        g_clear_pointer (&search->timestamp, g_date_time_unref);
 
         G_OBJECT_CLASS (gr_recipe_search_parent_class)->finalize (object);
 }
