@@ -229,13 +229,6 @@ gr_recipe_get_property (GObject    *object,
         }
 }
 
-static void
-update_mtime (GrRecipe *self)
-{
-        g_date_time_unref (self->mtime);
-        self->mtime = g_date_time_new_now_utc ();
-}
-
 static void
 set_images (GrRecipe *self,
             GArray   *images)
@@ -270,116 +263,108 @@ gr_recipe_set_property (GObject      *object,
         case PROP_ID:
                 g_free (self->id);
                 self->id = g_value_dup_string (value);
-                update_mtime (self);
                 break;
 
         case PROP_AUTHOR:
                 g_free (self->author);
                 self->author = g_value_dup_string (value);
-                update_mtime (self);
                 break;
 
         case PROP_NAME:
-                g_free (self->name);
-                g_free (self->cf_name);
+                g_clear_pointer (&self->name, g_free);
+                g_clear_pointer (&self->translated_name, g_free);
+                g_clear_pointer (&self->cf_name, g_free);
                 self->name = g_value_dup_string (value);
-                self->translated_name = translate_multiline_string (self->name);
-                self->cf_name = g_utf8_casefold (self->translated_name, -1);
-                update_mtime (self);
+                if (self->name) {
+                        self->translated_name = translate_multiline_string (self->name);
+                        self->cf_name = g_utf8_casefold (self->translated_name, -1);
+                }
                 break;
 
         case PROP_DESCRIPTION:
-                g_free (self->description);
-                g_free (self->cf_description);
+                g_clear_pointer (&self->description, g_free);
+                g_clear_pointer (&self->translated_description, g_free);
+                g_clear_pointer (&self->cf_description, g_free);
                 self->description = g_value_dup_string (value);
-                self->translated_description = translate_multiline_string (self->description);
-                self->cf_description = g_utf8_casefold (self->translated_description, -1);
-                update_mtime (self);
+                if (self->description) {
+                        self->translated_description = translate_multiline_string (self->description);
+                        self->cf_description = g_utf8_casefold (self->translated_description, -1);
+                }
                 break;
 
         case PROP_IMAGES:
                 set_images (self, (GArray *) g_value_get_boxed (value));
-                update_mtime (self);
                 break;
 
         case PROP_DEFAULT_IMAGE:
                 self->default_image = g_value_get_int (value);
-                update_mtime (self);
                 break;
 
         case PROP_CATEGORY:
                 g_free (self->category);
                 self->category = g_value_dup_string (value);
-                update_mtime (self);
                 break;
 
         case PROP_CUISINE:
                 g_free (self->cuisine);
                 self->cuisine = g_value_dup_string (value);
-                update_mtime (self);
                 break;
 
         case PROP_SEASON:
                 g_free (self->season);
                 self->season = g_value_dup_string (value);
-                update_mtime (self);
                 break;
 
         case PROP_PREP_TIME:
                 g_free (self->prep_time);
                 self->prep_time = g_value_dup_string (value);
-                update_mtime (self);
                 break;
 
         case PROP_COOK_TIME:
                 g_free (self->cook_time);
                 self->cook_time = g_value_dup_string (value);
-                update_mtime (self);
                 break;
 
         case PROP_SERVES:
                 self->serves = g_value_get_int (value);
-                update_mtime (self);
                 break;
 
         case PROP_SPICINESS:
                 self->spiciness = g_value_get_int (value);
-                update_mtime (self);
                 break;
 
         case PROP_INGREDIENTS:
-                {
-                g_autofree char *cf_garlic = NULL;
+                g_clear_pointer (&self->ingredients, g_free);
+                g_clear_pointer (&self->cf_ingredients, g_free);
+                self->garlic = FALSE;
 
-                g_free (self->ingredients);
-                g_free (self->cf_ingredients);
                 self->ingredients = g_value_dup_string (value);
-                self->cf_ingredients = g_utf8_casefold (self->ingredients, -1);
-
-                cf_garlic = g_utf8_casefold ("Garlic", -1);
-                self->garlic = (strstr (self->cf_ingredients, cf_garlic) != NULL);
-                update_mtime (self);
+                if (self->ingredients) {
+                        g_autofree char *cf_garlic = NULL;
+                        self->cf_ingredients = g_utf8_casefold (self->ingredients, -1);
+                        cf_garlic = g_utf8_casefold ("Garlic", -1);
+                        self->garlic = (strstr (self->cf_ingredients, cf_garlic) != NULL);
                 }
                 break;
 
         case PROP_INSTRUCTIONS:
-                g_free (self->instructions);
+                g_clear_pointer (&self->instructions, g_free);
+                g_clear_pointer (&self->translated_instructions, g_free);
                 self->instructions = g_value_dup_string (value);
-                self->translated_instructions = translate_multiline_string (self->instructions);
-                update_mtime (self);
+                if (self->instructions)
+                        self->translated_instructions = translate_multiline_string (self->instructions);
                 break;
 
         case PROP_NOTES:
-                g_free (self->notes);
-                g_free (self->translated_notes);
+                g_clear_pointer (&self->notes, g_free);
+                g_clear_pointer (&self->translated_notes, g_free);
                 self->notes = g_value_dup_string (value);
-                self->translated_notes = translate_multiline_string (self->notes);
-                update_mtime (self);
+                if (self->notes)
+                        self->translated_notes = translate_multiline_string (self->notes);
                 break;
 
         case PROP_DIETS:
                 self->diets = g_value_get_flags (value);
-                update_mtime (self);
                 break;
 
         case PROP_CTIME:
@@ -396,12 +381,10 @@ gr_recipe_set_property (GObject      *object,
 
         case PROP_READONLY:
                 self->readonly = g_value_get_boolean (value);
-                update_mtime (self);
                 break;
 
         case PROP_CONTRIBUTED:
                 self->contributed = g_value_get_boolean (value);
-                update_mtime (self);
                 break;
 
         default:
