@@ -55,9 +55,9 @@ struct _GrRecipesPage
 
 G_DEFINE_TYPE (GrRecipesPage, gr_recipes_page, GTK_TYPE_BOX)
 
-static void populate_diets_from_store (GrRecipesPage *page);
 static void populate_recipes_from_store (GrRecipesPage *page);
 static void populate_shopping_from_store (GrRecipesPage *page);
+static void populate_categories_from_store (GrRecipesPage *page);
 static void populate_chefs_from_store (GrRecipesPage *page);
 static void connect_store_signals (GrRecipesPage *page);
 
@@ -140,9 +140,9 @@ gr_recipes_page_init (GrRecipesPage *page)
         gtk_widget_set_has_window (GTK_WIDGET (page), FALSE);
         gtk_widget_init_template (GTK_WIDGET (page));
 
-        populate_diets_from_store (page);
         populate_recipes_from_store (page);
         populate_shopping_from_store (page);
+        populate_categories_from_store (page);
         populate_chefs_from_store (page);
         gr_recipe_tile_recreate_css ();
         gr_chef_tile_recreate_css ();
@@ -209,10 +209,14 @@ category_clicked (GrCategoryTile *tile,
                 gr_window_show_favorites (GR_WINDOW (window));
         else if (strcmp (name, "mine") == 0)
                 gr_window_show_myself (GR_WINDOW (window));
+        else if (strcmp (name, "all") == 0)
+                gr_window_show_all (GR_WINDOW (window));
+        else if (strcmp (name, "new") == 0)
+                gr_window_show_new (GR_WINDOW (window));
 }
 
 static void
-populate_diets_from_store (GrRecipesPage *self)
+populate_categories_from_store (GrRecipesPage *self)
 {
         int i;
         GrDiets diets[5] = {
@@ -237,12 +241,22 @@ populate_diets_from_store (GrRecipesPage *self)
         gtk_container_add (GTK_CONTAINER (self->diet_box), tile);
         g_signal_connect (tile, "clicked", G_CALLBACK (category_clicked), self);
 
+        tile = gr_category_tile_new_with_label ("all", _("All Recipes"));
+        gtk_widget_show (tile);
+        gtk_container_add (GTK_CONTAINER (self->diet_box), tile);
+        g_signal_connect (tile, "clicked", G_CALLBACK (category_clicked), self);
+
+        tile = gr_category_tile_new_with_label ("new", _("New Recipes"));
+        gtk_widget_show (tile);
+        gtk_container_add (GTK_CONTAINER (self->diet_box), tile);
+        g_signal_connect (tile, "clicked", G_CALLBACK (category_clicked), self);
+
         for (i = 0; i < G_N_ELEMENTS (diets); i++) {
                 tile = gr_category_tile_new (diets[i]);
                 gtk_widget_show (tile);
                 g_signal_connect (tile, "clicked", G_CALLBACK (category_clicked), self);
 
-                if (i < 4)
+                if (i + 4 < 6)
                         gtk_container_add (GTK_CONTAINER (self->diet_box), tile);
                 else
                         gtk_container_add (GTK_CONTAINER (self->diet_box2), tile);
