@@ -39,6 +39,9 @@ void            gr_window_show_recipe                (GrWindow   *window,
                                                       GrRecipe   *recipe);
 void            gr_window_edit_recipe                (GrWindow   *window,
                                                       GrRecipe   *recipe);
+void            gr_window_timer_expired              (GrWindow   *window,
+                                                      GrRecipe   *recipe,
+                                                      int         step);
 void            gr_window_load_recipe                (GrWindow   *window,
                                                       GFile      *file);
 void            gr_window_show_search                (GrWindow   *window,
@@ -51,6 +54,8 @@ void            gr_window_show_chef                  (GrWindow   *window,
                                                       GrChef     *chef);
 void            gr_window_show_myself                (GrWindow   *window);
 void            gr_window_show_favorites             (GrWindow   *window);
+void            gr_window_show_all                   (GrWindow   *window);
+void            gr_window_show_new                   (GrWindow   *window);
 void            gr_window_show_list                  (GrWindow   *window,
                                                       const char *title,
                                                       GList      *recipes);
